@@ -7,6 +7,7 @@ import click
 from dtale.cli.loaders import (
     arctic_loader,
     csv_loader,
+    excel_loader,
     json_loader,
     parquet_loader,
     sqlite_loader,
@@ -44,16 +45,6 @@ def get_py33_loader(fname, path):
     return SourceFileLoader(*build_custom_module_loader_args(fname, path)).load_module()
 
 
-def get_py2_loader(fname, path):
-    """
-    Utility function for loading dynamic modules (CLI configurations) when python_version < 3
-
-    """
-    import imp
-
-    return imp.load_source(*build_custom_module_loader_args(fname, path))
-
-
 def unsupported_python_version(version_tuple):
     return (
         "Unsupported version of python used for custom CLI loaders, {}. If you do not plan on using any custom "
@@ -70,8 +61,6 @@ def custom_module_loader():
 
     """
     major, minor, revision = [int(i) for i in platform.python_version_tuple()]
-    if major == 2:
-        return get_py2_loader
     if major == 3:
         if minor >= 5:
             return get_py35_loader
@@ -83,6 +72,7 @@ def custom_module_loader():
 LOADERS = {
     arctic_loader.LOADER_KEY: arctic_loader,
     csv_loader.LOADER_KEY: csv_loader,
+    excel_loader.LOADER_KEY: excel_loader,
     json_loader.LOADER_KEY: json_loader,
     parquet_loader.LOADER_KEY: parquet_loader,
     sqlite_loader.LOADER_KEY: sqlite_loader,
